from typing import Dict, List, Any
import json, os

import bottle
from configparser import ConfigParser
from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class Halyk(metaclass=Singleton):
    log: Logger
    cases: Dict[str, List[Dict[str, Any]]]

    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса')
        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'cases.json'), 'r') as cases:
                self.cases = json.load(cases)
        except Exception as e:
            self.log.error(str(e))

        conf = ConfigParser()
        conf.read(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'config.ini'))
        servicePath = conf.get('halyk', 'path') if conf.has_option('halyk', 'path') else '/halyk'

        servicestub.route(servicePath, method='POST', callback=self.halyk)

    def halyk(self):
        body = json.loads(bottle.request.body.read())
        self.log.info("Данные запроса '%s'" % str(body))

        if 'task' not in body:
            raise bottle.HTTPError(400, 'В запросе нет поля task')

        taskName = body['task']
        if taskName not in self.cases:
            raise bottle.HTTPError(400, 'Метод "%s" не найден' % taskName)

        if not len(self.cases[taskName]):
            self.log.warn('В кейсах не осталось значений')
            return None

        # если в кейсе нет счётчика, то возвращаем str(response)
        if 'count' not in self.cases[taskName][0]:
            return json.dumps(self.cases[taskName][0].get('response'))

        self.cases[taskName][0]['count'] -= 1
        response = json.dumps(self.cases[taskName][0].get('response'))

        if self.cases[taskName][0]['count'] == 0:
            del self.cases[taskName][0]

        return response


instance = Halyk()
